#! /usr/bin/python3
#-------------------------------------------------------------------------------
#
# Copyright 2015, 2020 Cumulus Networks, inc  all rights reserved
#
#-------------------------------------------------------------------------------
#

try:
    import os
    import threading
except ImportError as e:
    raise ImportError (str(e) + "- required module not found")

class ClagCondition(threading.Condition):
    def __init__(self, lock=None, verbose=None):
        threading.Condition.__init__(self, lock)

    def wait(self, timeout=None):
        '''
        This is a direct copy of the Condition class in the standard threading
        library, except the _time() functions have been replaced by os.times()
        calls so that changes in the system date/time do not affect time which
        we wait for in this function.
        '''
        if not self._is_owned():
            raise RuntimeError("cannot wait on un-acquired lock")
        waiter = threading._allocate_lock()
        waiter.acquire()
        self.Condition__waiters.append(waiter)
        saved_state = self._release_save()
        try:    # restore state no matter what (e.g., KeyboardInterrupt)
            if timeout is None:
                waiter.acquire()
                if __debug__:
                    self._note("%s.wait(): got it", self)
            else:
                endtime = os.times()[4] + timeout
                delay = 0.005 # 5 ms -> initial delay of 10 ms
                while True:
                    gotit = waiter.acquire(0)
                    if gotit:
                        break
                    remaining = endtime - os.times()[4]
                    if remaining <= 0:
                        break
                    delay = min(delay * 2, remaining, .05)
                    threading._sleep(delay)
                if not gotit:
                    if __debug__:
                        self._note("%s.wait(%s): timed out", self, timeout)
                    try:
                        self.Condition__waiters.remove(waiter)
                    except ValueError:
                        pass
                else:
                    if __debug__:
                        self._note("%s.wait(%s): got it", self, timeout)
        finally:
            self._acquire_restore(saved_state)

class Event(threading.Event):
    """
    This class simply overrides the __cond member of the standard threading
    Event class to use the clag alternative.
    """
    def __init__(self, verbose=None):
        threading.Event.__init__(self)
        self.Event__cond = ClagCondition(threading.Lock())


